import ctypes
from datetime import datetime
import json
import logging
import os
import platform
import re
import stat
from pathlib import Path
from typing import Optional, List, Set, Union, Dict, Any, Tuple, cast

from cachetools import cached, TTLCache
from pydantic import ValidationError

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.severities import get_severity, Severity, Severities, BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.report import Report
from checkov.common.sast.consts import SastLanguages
from checkov.common.sca.reachability.sast_contract.data_fetcher_sast_lib import SastReachabilityDataFetcher
from checkov.common.typing import _CheckResult
from checkov.common.util.http_utils import request_wrapper
from checkov.sast.checks_infra.base_registry import Registry
from checkov.sast.common import get_code_block_from_start, get_data_flow_code_block
from checkov.sast.engines.base_engine import SastEngine
from checkov.sast.prisma_models.library_input import LibraryInput
from checkov.sast.prisma_models.policies_list import SastPolicies
from checkov.sast.prisma_models.report import PrismaReport, create_empty_report
from checkov.sast.record import SastRecord
from checkov.sast.report import SastReport

logger = logging.getLogger(__name__)

REPORT_PARSING_ERRORS = "report_parsing_errors"
FILE_NAME_PATTERN = re.compile(r"(\d+_\d+_\d+)_library\.(so|dll|dylib)")
SAST_CORE_FILENAME_PATTERN = re.compile(rf"{FILE_NAME_PATTERN.pattern}$")
SAST_CORE_URL_PATTERN = re.compile(rf".*/(?P<name>v?{FILE_NAME_PATTERN.pattern})\?.*")


class PrismaEngine(SastEngine):
    def __init__(self) -> None:
        self.lib_path = ""
        self.check_type = CheckType.SAST
        self.prisma_sast_dir_path = Path(bridgecrew_dir) / "sast"
        self.sast_platform_base_path = "api/v1/sast"

    def get_check_thresholds(self, registry: Registry) -> Tuple[Severity, Severity]:
        """
        Returns a tuple of check threshold and skip check threshold..

        If a severity was specified in --check and / or --skip-check, then return a tuple of those values (these override enforcement rules).
        Else if enforcement rules are enabled, return a tuple of the enforcement rule's SAST soft fail threshold and NONE.
        Else return a tuple of NONE, NONE
        """
        none = Severities[BcSeverities.NONE]

        check_threshold: Optional[Severity] = registry.runner_filter.check_threshold  # type:ignore[union-attr] # not null
        skip_check_threshold: Optional[Severity] = registry.runner_filter.skip_check_threshold  # type:ignore[union-attr] # not null
        enforcement_threshold: Optional[Severity] = cast(Severity, registry.runner_filter.enforcement_rule_configs[self.check_type]) if registry.runner_filter.use_enforcement_rules else None  # type:ignore[union-attr] # not null

        return (check_threshold or none, skip_check_threshold or none) if (check_threshold or skip_check_threshold) else \
            (enforcement_threshold, none) if enforcement_threshold else \
            (none, none)

    def get_reports(self, targets: List[str], registry: Registry, languages: Set[SastLanguages]) -> List[Report]:
        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run SAST Prisma Cloud scanning")
            return []

        status = self.setup_sast_artifact()
        if not status:
            return []

        prisma_lib_path = self.get_sast_artifact()
        if not prisma_lib_path:
            return []

        self.lib_path = str(prisma_lib_path)

        check_threshold, skip_check_threshold = self.get_check_thresholds(registry)

        library_input: LibraryInput = {
            'languages': languages,
            'source_codes': targets,
            'policies': registry.checks_dirs_path,
            'checks': registry.runner_filter.checks if registry.runner_filter else [],
            'skip_checks': registry.runner_filter.skip_checks if registry.runner_filter else [],
            'check_threshold': check_threshold,
            'skip_check_threshold': skip_check_threshold,
            'skip_path': registry.runner_filter.excluded_paths if registry.runner_filter else [],
            'report_imports': registry.runner_filter.report_sast_imports if registry.runner_filter else False,
            'remove_default_policies': registry.runner_filter.remove_default_sast_policies if registry.runner_filter else False,
            'report_reachability': registry.runner_filter.report_sast_reachability if registry.runner_filter else False,
        }
        prisma_result = self.run_go_library(**library_input)

        return prisma_result

    def setup_sast_artifact(self) -> bool:
        current_version = ""
        if not self.prisma_sast_dir_path.exists():
            try:
                os.makedirs(self.prisma_sast_dir_path, exist_ok=True)
            except FileExistsError:
                pass
        else:
            is_file_exists = [f for f in os.listdir(self.prisma_sast_dir_path) if
                              (self.prisma_sast_dir_path / f).is_file() and "library" in f]
            if len(is_file_exists) > 0:
                latest_file = os.path.join(self.prisma_sast_dir_path, is_file_exists[0])
                creation_time = os.path.getmtime(latest_file)
                now = datetime.now().timestamp()
                diff = datetime.fromtimestamp(now) - datetime.fromtimestamp(creation_time)
                if diff.days < 1:
                    match = re.search(SAST_CORE_FILENAME_PATTERN, latest_file)
                    if match:
                        current_version = match.groups()[0]

        if os.getenv("SAST_ARTIFACT_PATH"):
            logging.debug(f'using local artifact in path {os.getenv("SAST_ARTIFACT_PATH")}')
            return True
        status: bool = self.download_sast_artifacts(current_version)

        return status

    @cached(TTLCache(maxsize=1, ttl=300))
    def download_sast_artifacts(self, current_version: str) -> bool:
        try:
            machine = get_machine()
            os_type = platform.system().lower()
            headers = bc_integration.get_default_headers("GET")
            headers["X-Client-Sast-Version"] = current_version
            headers["X-Required-Sast-Version"] = "latest"  # or ant version seperated with _

            # don't use the 'should_call_raise_for_status' parameter for now, because it logs errors messages
            response = request_wrapper(
                method="GET",
                url=f"{bc_integration.api_url}/{self.sast_platform_base_path}/{os_type}/{machine}/artifacts",
                headers=headers,
            )
            response.raise_for_status()

            if response.status_code == 304:
                return True

            match = re.match(SAST_CORE_URL_PATTERN, response.url)
            if match:
                new_name = match.group('name')
                cli_file_name_path = self.prisma_sast_dir_path / new_name
                self._cleanup_scan()
                cli_file_name_path.touch(exist_ok=True)
                cli_file_name_path.write_bytes(response.content)
                cli_file_name_path.chmod(cli_file_name_path.stat().st_mode | stat.S_IEXEC | stat.S_IREAD)
                logging.debug("sast artifacts downloaded")
            return True
        except Exception:
            logging.debug(
                "Unexpected failure happened during downloading sast artifact. details are below.\n"
                "scanning is terminating. please try again. if it is repeated, please report.\n", exc_info=True)
            return False

    def _cleanup_scan(self) -> None:
        if self.prisma_sast_dir_path.exists():
            for file in os.scandir(self.prisma_sast_dir_path):
                os.unlink(file.path)
            # shutil.rmtree(self.prisma_sast_dir_path)
            logging.info('sast dir is clear')
        else:
            self.prisma_sast_dir_path.mkdir(exist_ok=True)

    def get_sast_artifact(self) -> Optional[Path]:
        env_variable_path = os.getenv("SAST_ARTIFACT_PATH")
        if env_variable_path and os.path.isfile(env_variable_path):
            return Path(env_variable_path)

        files = [(self.prisma_sast_dir_path / f) for f in os.listdir(self.prisma_sast_dir_path) if
                 (self.prisma_sast_dir_path / f).is_file() and "library" in f]

        if len(files) == 0:
            return None

        return files[0]

    def run_go_library(self, languages: Set[SastLanguages],
                       source_codes: List[str],
                       policies: List[str],
                       checks: List[str],
                       skip_checks: List[str],
                       skip_path: List[str],
                       check_threshold: Severity,
                       skip_check_threshold: Severity,
                       list_policies: bool = False,
                       report_imports: bool = True,
                       report_reachability: bool = False,
                       remove_default_policies: bool = False) -> Union[List[Report], SastPolicies]:

        validate_params(languages, source_codes, list_policies)

        if bc_integration.bc_source:
            name = bc_integration.bc_source.name
        else:
            name = "unknown"

        reachability_data = None
        if report_reachability or report_imports:
            # TODO - run sast-core per src
            for source_code in source_codes:
                reachability_data = get_reachability_data(source_code)

        document = {
            "scan_code_params": {
                "source_codes": source_codes,
                "policies": policies,
                "languages": [a.value for a in languages],
                "checks": checks,
                "skip_checks": skip_checks,
                "skip_path": skip_path,
                "check_threshold": str(check_threshold),
                "skip_check_threshold": str(skip_check_threshold),
                "list_policies": list_policies,
                "report_imports": report_imports,
                "remove_default_policies": remove_default_policies,
                "report_reachability": report_reachability,
                "reachability_data": reachability_data
            },
            "auth": {
                "api_key": bc_integration.get_auth_token(),
                "platform_url": bc_integration.api_url,
                "client_name": name,
                "version": bc_integration.bc_source_version
            }
        }

        if list_policies:
            return self.run_go_library_list_policies(document)

        library = ctypes.cdll.LoadLibrary(self.lib_path)
        analyze_code = library.analyzeCode
        analyze_code.restype = ctypes.c_void_p

        # send the document as a byte array of json format
        analyze_code_output = analyze_code(json.dumps(document).encode('utf-8'))

        # we dereference the pointer to a byte array
        analyze_code_bytes = ctypes.string_at(analyze_code_output)

        # convert our byte array to a string
        analyze_code_string = analyze_code_bytes.decode('utf-8')
        d = json.loads(analyze_code_string)

        try:
            result = self.create_prisma_report(d)
        except ValidationError as e:
            result = create_empty_report(list(languages))
            result.errors = {REPORT_PARSING_ERRORS: [str(err) for err in e.errors()]}
        return self.create_report(result)

    def create_prisma_report(self, data: Dict[str, Any]) -> PrismaReport:
        if not data.get("imports"):
            data["imports"] = {}
        if not data.get("reachability_report"):
            data["reachability_report"] = {}
        return PrismaReport(**data)

    def run_go_library_list_policies(self, document: Dict[str, Any]) -> SastPolicies:
        try:
            library = ctypes.cdll.LoadLibrary(self.lib_path)
            list_policies = library.listPolicies
            list_policies.restype = ctypes.c_void_p

            # send the document as a byte array of json format
            list_policies_output = list_policies(json.dumps(document).encode('utf-8'))

            # we dereference the pointer to a byte array
            list_policies_bytes = ctypes.string_at(list_policies_output)

            # convert our byte array to a string
            list_policies_string = list_policies_bytes.decode('utf-8')

            d = json.loads(list_policies_string)
        except Exception as e:
            logging.error(e)
            return {}

        try:
            return SastPolicies(**d)
        except ValidationError:
            if d.get('errors'):
                logging.error(d.get('errors'))
            return {}

    def create_report(self, prisma_report: PrismaReport) -> List[SastReport]:
        logging.debug("Printing Prisma-SAST profiling data")
        logging.debug(prisma_report.profiler)
        reports: List[SastReport] = []
        for lang, checks in prisma_report.rule_match.items():
            report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang)
            for check_id, match_rule in checks.items():
                check_name = match_rule.check_name
                check_cwe = match_rule.check_cwe
                check_owasp = "TBD"  # match.metadata.get('owasp')
                check_result = _CheckResult(result=CheckResult.FAILED)
                severity = get_severity(match_rule.severity)

                for match in match_rule.matches:
                    location = match.location
                    file_abs_path = location.path
                    file_path = file_abs_path.split('/')[-1]
                    file_line_range = [location.start.row, location.end.row]

                    if match.metadata.taint_mode is not None:
                        code_block = get_data_flow_code_block(match.metadata.taint_mode.data_flow)
                    else:
                        split_code_block = [line + '\n' for line in location.code_block.split('\n')]
                        code_block = get_code_block_from_start(split_code_block, location.start.row)

                    record = SastRecord(check_id=check_id, check_name=check_name, resource="", evaluations={},
                                        check_class="", check_result=check_result, code_block=code_block,
                                        file_path=file_path, file_line_range=file_line_range,
                                        file_abs_path=file_abs_path, severity=severity, cwe=check_cwe,
                                        owasp=check_owasp, show_severity=True)
                    report.add_record(record)
            report_parsing_errors = prisma_report.errors.get(REPORT_PARSING_ERRORS)
            if report_parsing_errors:
                report.add_parsing_errors(report_parsing_errors)
            reports.append(report)

        for lang in prisma_report.imports:
            for report in reports:
                if report.language == lang:
                    report.sast_imports = prisma_report.imports[lang]
                    break
            else:
                report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang)
                report.sast_imports = prisma_report.imports[lang]
                reports.append(report)

        for lang in prisma_report.reachability_report:
            for report in reports:
                if report.language == lang:
                    report.sast_reachability = prisma_report.reachability_report[lang]
                    break
            else:
                report = SastReport(f'{self.check_type.lower()}_{lang.value}', prisma_report.run_metadata, lang)
                report.sast_reachability = prisma_report.reachability_report[lang]
                reports.append(report)
        return reports

    def get_policies(self, languages: Set[SastLanguages]) -> SastPolicies:
        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run Sast prisma scanning")
            return []

        self.setup_sast_artifact()
        prisma_lib_path = self.get_sast_artifact()
        if not prisma_lib_path:
            return []

        self.lib_path = str(prisma_lib_path)

        library_input: LibraryInput = {
            'languages': languages,
            'list_policies': True,
            'source_codes': [],
            'policies': [],
            'checks': [],
            'skip_checks': [],
            'check_threshold': Severities[BcSeverities.NONE],
            'skip_check_threshold': Severities[BcSeverities.NONE],
            'skip_path': [],
            'report_imports': False,
            'report_reachability': False
        }
        prisma_result = self.run_go_library(**library_input)
        return prisma_result


def validate_params(languages: Set[SastLanguages],
                    source_codes: List[str],
                    list_policies: bool) -> None:
    if list_policies:
        return

    if len(source_codes) == 0:
        raise Exception('must provide source code file or dir for sast runner')

    if len(languages) == 0:
        raise Exception('must provide a language for sast runner')


def get_machine() -> str:
    machine = platform.machine().lower()
    if machine in ['amd64', 'x86', 'x86_64', 'x64']:
        return "amd64"

    if machine in ['arm', 'arm64', 'armv8', 'aarch64', 'arm64-v8a']:
        return 'arm64'

    return ''


def get_reachability_data(repo_path: str) -> Dict[str, Any]:
    fetcher = SastReachabilityDataFetcher()
    reachability_data = fetcher.fetch(repository_name=repo_path, repository_root_dir=repo_path)
    data: Dict[str, Any] = {}
    if not reachability_data:
        return data
    langs = reachability_data.aliasMapping.get("languages")
    if not langs:
        return {}
    for lang, lang_data in langs.items():
        if lang == "nodejs":
            lang = "javascript"
        data[lang] = {"package_alias": {}}
        for _, files in lang_data.get("repositories", {}).items():
            for _, files_data in files.get("files", {}).items():
                for original_package_name, package_alias in files_data.get("packageAliases", {}).items():
                    aliases = package_alias.get("packageAliases", [])
                    if aliases:
                        data[lang]["package_alias"][original_package_name] = aliases[0]
    return data



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class WAFHasLogs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Logging is enabled for WAF  Web Access Control Lists"
        id = "CKV_AWS_176"
        supported_resources = ('aws_waf_web_acl', 'aws_wafregional_web_acl')
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'logging_configuration/[0]/log_destination'

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = WAFHasLogs()



from __future__ import annotations

import json
import logging
import os
from pathlib import Path
from typing import Any, TYPE_CHECKING

import yaml

from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.graph.checks_infra.base_parser import BaseGraphCheckParser
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.runner_filter import RunnerFilter
from checkov.common.checks_infra.resources_types import resources_types

if TYPE_CHECKING:
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck

CHECKS_POSSIBLE_ENDING = {".json", ".yaml", ".yml"}


class Registry(BaseRegistry):
    def __init__(self, checks_dir: str, parser: BaseGraphCheckParser | None = None) -> None:
        parser = parser or BaseGraphCheckParser()

        super().__init__(parser)
        self.checks: list[BaseGraphCheck] = []
        self.checks_dir = checks_dir
        self.logger = logging.getLogger(__name__)
        add_resource_code_filter_to_logger(self.logger)

    def load_checks(self) -> None:
        if self.checks:
            # checks were previously loaded
            return

        self._load_checks_from_dir(self.checks_dir, False)

    def _load_checks_from_dir(self, directory: str, external_check: bool) -> None:
        dir = os.path.expanduser(directory)
        self.logger.debug(f"Loading external checks from {dir}")
        for root, d_names, f_names in os.walk(dir):
            self.logger.debug(f"Searching through {d_names} and {f_names}")
            for file in f_names:
                file_ending = os.path.splitext(file)[1]
                if file_ending in CHECKS_POSSIBLE_ENDING:
                    with open(os.path.join(root, file), "r") as f:
                        if dir != self.checks_dir:
                            self.logger.info(f"loading {file}")

                        if file_ending == ".json":
                            check_json = json.load(f)
                        else:
                            check_yaml = yaml.safe_load(f)
                            check_json = json.loads(json.dumps(check_yaml))

                        if not isinstance(check_json, dict):
                            self.logger.error(f"Loaded data from JSON is not Dict. Skipping. Data: {check_json}.")
                            continue

                        if not self.parser.validate_check_config(file_path=f.name, raw_check=check_json):
                            # proper log messages are generated inside the method
                            continue

                        check = self.parser.parse_raw_check(
                            check_json, resources_types=self._get_resource_types(check_json), check_path=f'{root}/{file}'
                        )
                        if not any(c for c in self.checks if check.id == c.id):
                            if external_check:
                                # Note the external check; used in the should_run_check logic
                                RunnerFilter.notify_external_check(check.id)
                            self.checks.append(check)

    def load_external_checks(self, dir: str) -> None:
        self._load_checks_from_dir(dir, True)

    @staticmethod
    def _get_resource_types(check_json: dict[str, dict[str, Any]]) -> list[str] | None:
        provider = check_json.get("scope", {}).get("provider", "").lower()
        return resources_types.get(provider)


_registry_instances: dict[str, Registry] = {}


def get_graph_checks_registry(check_type: str) -> Registry:
    if not _registry_instances.get(check_type):
        _registry_instances[check_type] = Registry(
            parser=GraphCheckParser(),
            checks_dir=f"{Path(__file__).parent.parent.parent}/{check_type}/checks/graph_checks",
        )
    return _registry_instances[check_type]



from abc import ABC, abstractmethod
import logging
import os
import re
from typing import Tuple, Dict, Any, List

import dpath

from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger

TF_DEFINITIONS_STRIP_WORDS = re.compile(r"\b(?!\d)([^\/]+)")
NON_PATH_WORDS_REGEX = re.compile(r"\b(?!output)[^ .]+")
DEFINITION_TYPES_REGEX_MAPPING = {"variable": "var", "locals": "local"}


class BaseVariableEvaluation(ABC):
    def __init__(
        self,
        root_folder: str,
        tf_definitions: Dict[str, Dict[str, Any]],
        definitions_context: Dict[str, Dict[str, Any]],
    ) -> None:
        self.logger = logging.getLogger("{}".format(self.__module__))
        add_resource_code_filter_to_logger(self.logger)
        self.root_folder = root_folder
        self.tf_definitions = tf_definitions
        self.definitions_context = definitions_context

    @abstractmethod
    def evaluate_variables(self) -> Any:
        """
        evaluate variables of tf_definitions entities
        :return:
        """
        raise NotImplementedError()

    @staticmethod
    def extract_context_path(definition_path: str) -> Tuple[str, str]:
        """
        Converts a JSONPath (dpath library standard) definition entry path into valid context parser path
        :param definition_path: entity's JSONPath syntax path in tf_definitions
        :return:entity path in context parser
        """
        return os.path.split("/".join(re.findall(TF_DEFINITIONS_STRIP_WORDS, definition_path)))

    @staticmethod
    def reduce_entity_evaluations(
        variables_evaluations: Dict[str, Dict[str, Any]], entity_context_path: List[str]
    ) -> Dict[str, Any]:
        """
        Reduce variable evaluations only to variables that are included in the entity's code block
        :param variables_evaluations:
        :param entity_context_path:
        :return: the variable evaluations of the entity
        """
        entity_evaluations: Dict[str, Any] = {}
        for var_name, variable_evaluations in variables_evaluations.items():
            entity_definitions = []
            for var_definition in variable_evaluations["definitions"]:
                var_context_path, _ = BaseVariableEvaluation.extract_context_path(var_definition["definition_path"])
                variable_context_path = var_context_path.split("/")
                # This is due to inconsistency in order of Terraform entity naming conventions
                if set(variable_context_path) == set(entity_context_path):
                    entity_definitions.append(var_definition)
            if entity_definitions:
                entity_evaluation = variables_evaluations[var_name]
                entity_evaluation["definitions"] = entity_definitions
                dpath.new(entity_evaluations, var_name, entity_evaluation)
        return entity_evaluations



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck


class SecurityOperations(BaseOpenapiCheck):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_5"
        name = "Ensure that security operations is not empty."
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ['security']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        self.evaluated_keys = ['security', 'paths']

        # Check if security field is present and not empty at the root level
        root_security = conf.get('security')

        # If security field is not present or empty at the root level, check within each operation
        paths = conf.get('paths', {}) or {}
        if isinstance(paths, dict):
            for path, http_method in paths.items():
                if self.is_start_end_line(path):
                    continue
                if isinstance(http_method, dict):
                    for op_name, op_val in http_method.items():
                        if self.is_start_end_line(op_name):
                            continue
                        self.evaluated_keys = ['security']
                        if not isinstance(op_val, dict):
                            continue
                        op_security = op_val.get("security")
                        if op_security is not None and not op_security:
                            # fails when security field is set as empty list
                            return CheckResult.FAILED, conf

                        if op_security is None and not root_security:
                            # no security field for the operation and not in the root
                            return CheckResult.FAILED, conf

        return CheckResult.PASSED, conf


check = SecurityOperations()



#!/usr/bin/env python
import json
import logging
import os
from importlib import util
from os import path
from pathlib import Path

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py


class PreBuildCommand(build_py):
    """Pre-build command"""

    def transform_graph_yaml_to_json(self) -> None:
        """Transforms YAML graph checks to JSON and copies them to build/lib"""

        import yaml  # can't be top-level, because it needs to be first installed via 'setup_requires'

        graph_check_paths = ("checkov/*/checks/graph_checks",)
        build_path = Path(self.build_lib)
        src_path = Path()

        for graph_check_path in graph_check_paths:
            for yaml_file in src_path.glob(f"{graph_check_path}/**/*.yaml"):
                json_file = (build_path / yaml_file).with_suffix(".json")
                self.mkpath(str(json_file.parent))
                json_file.write_text(json.dumps(yaml.safe_load(yaml_file.read_text())))

    def run(self) -> None:
        self.execute(self.transform_graph_yaml_to_json, ())
        build_py.run(self)


# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

logger = logging.getLogger(__name__)
spec = util.spec_from_file_location(
    "checkov.version", os.path.join("checkov", "version.py")
)
# noinspection PyUnresolvedReferences
mod = util.module_from_spec(spec)
spec.loader.exec_module(mod)  # type: ignore
version = mod.version  # type: ignore

setup(
    cmdclass={
        "build_py": PreBuildCommand,
    },
    setup_requires=[
        "pyyaml",
    ],
    extras_require={
        "dev": [
            "pytest==5.3.1",
            "coverage==5.5",
            "coverage-badge",
            "GitPython==3.1.7",
            "bandit",
            "jsonschema",
        ]
    },
    install_requires=[
        "bc-python-hcl2==0.4.1",
        "bc-detect-secrets==1.4.30",
        "bc-jsonpath-ng==1.6.1",
        "tabulate",
        "colorama",
        "termcolor",
        "junit-xml>=1.9",
        "dpath==2.1.3",
        "pyyaml>=5.4.1",
        "boto3>=1.17",
        "gitpython",
        "jmespath",
        "tqdm",
        "update-checker",
        "semantic-version",
        "packaging",
        "cloudsplaining>=0.6.2",
        "networkx<2.7",
        "igraph<0.11.0",
        "dockerfile-parse",
        "docker",
        "configargparse",
        "argcomplete",
        "typing-extensions>=4.1.0",
        "importlib-metadata>=0.12",
        "cachetools",
        "cyclonedx-python-lib>=5.0.0",
        "packageurl-python",
        "click>=8.0.0",
        "aiohttp",
        "aiodns",
        "aiomultiprocess",
        "jsonschema>=4.6.0,<5.0.0",
        "prettytable>=3.0.0",
        "pycep-parser==0.4.1",
        "charset-normalizer",
        "pyston-autoload==2.3.5; python_version < '3.11' and (sys_platform == 'linux' or sys_platform == 'darwin') and platform_machine == 'x86_64' and implementation_name == 'cpython'",
        "pyston==2.3.5; python_version < '3.11' and (sys_platform == 'linux' or sys_platform == 'darwin') and platform_machine == 'x86_64' and implementation_name == 'cpython'",
        "schema",
        "requests>=2.27.0",
        "yarl",
        "openai<1.0.0",
        "spdx-tools<0.9.0,>=0.8.0",
        "license-expression",
        "rustworkx",
        "pydantic",
    ],
    dependency_links=[],  # keep it empty, needed for pipenv-setup
    license="Apache License 2.0",
    name="checkov",
    version=version,
    python_requires=">=3.8",
    description="Infrastructure as code static analysis",
    author="bridgecrew",
    author_email="meet@bridgecrew.io",
    url="https://github.com/bridgecrewio/checkov",
    packages=find_packages(
        exclude=[
            "dogfood_tests*",
            "flake8_plugins*",
            "integration_tests*",
            "performance_tests*",
            "tests*",
        ]
    ),
    include_package_data=True,
    package_data={
        "checkov": ["py.typed"],
        "checkov.common.util.templates": ["*.jinja2"],
        "checkov.ansible.checks.graph_checks": ["**/*.json"],
        "checkov.arm.checks.graph_checks": ["**/*.json"],
        "checkov.bicep.checks.graph_checks": ["**/*.json"],
        "checkov.cloudformation.checks.graph_checks": ["**/*.json"],
        "checkov.dockerfile.checks.graph_checks": ["**/*.json"],
        "checkov.github_actions.checks.graph_checks": ["**/*.json"],
        "checkov.kubernetes.checks.graph_checks": ["**/*.json"],
        "checkov.terraform.checks.graph_checks": ["**/*.json"],
        "checkov.sast.checks": [
            "java/*.yaml",
            "python/*.yaml",
            "javascript/*.yaml",
        ],
    },
    scripts=["bin/checkov", "bin/checkov.cmd"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Security",
        "Topic :: Software Development :: Build Tools",
        "Typing :: Typed",
    ],
)



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class OperationObjectConsumesUndefined(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_17"
        name = "Ensure that operation objects have 'consumes' field defined for PUT, POST and PATCH operations - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["paths"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        paths = conf.get('paths') or {}

        for path, path_dict in paths.items():
            if self.is_start_end_line(path):
                continue
            if not isinstance(path_dict, dict):
                return CheckResult.UNKNOWN, conf
            for operation, operation_dict in path_dict.items():
                if self.is_start_end_line(operation):
                    continue
                if operation.lower() in ['post', 'put', 'patch']:
                    if not isinstance(operation_dict, dict):
                        return CheckResult.UNKNOWN, conf
                    if not operation_dict.get('consumes'):
                        return CheckResult.FAILED, operation_dict

        return CheckResult.PASSED, conf


check = OperationObjectConsumesUndefined()



from __future__ import annotations

from typing import Any

from checkov.ansible.checks.base_ansible_task_value_check import BaseAnsibleTaskValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class EC2EBSOptimized(BaseAnsibleTaskValueCheck):
    def __init__(self) -> None:
        name = "Ensure that EC2 is EBS optimized"
        id = "CKV_AWS_135"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.GENERAL_SECURITY,),
            supported_modules=("amazon.aws.ec2_instance", "ec2_instance"),
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not conf.get("image_id") and not conf.get("image"):
            # if 'image_id' or 'image' are not set, then an already running instance is targeted
            return CheckResult.UNKNOWN, self.entity_conf

        return super().scan_conf(conf=conf)

    def get_inspected_key(self) -> str:
        return "ebs_optimized"


check = EC2EBSOptimized()




from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureFirewallPolicyIDPSDeny(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Firewall policy has IDPS mode as deny"
        id = "CKV_AZURE_220"
        supported_resources = ['azurerm_firewall_policy']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "intrusion_detection/[0]/mode"

    def get_expected_value(self) -> str:
        return "Deny"


check = AzureFirewallPolicyIDPSDeny()



import re
from typing import Any, Dict

from checkov.common.models.consts import DOCKER_IMAGE_REGEX
from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ImageTagFixed(BaseK8sContainerCheck):
    def __init__(self) -> None:
        """
        You should avoid using the :latest tag when deploying containers in production
        as it is harder to track which version of the image is running
        and more difficult to roll back properly.
        """
        name = "Image Tag should be fixed - not latest or blank"
        id = "CKV_K8S_14"
        # Location: container .image
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["image"]
        if conf.get("image"):

            image_val = conf["image"]
            if not isinstance(image_val, str) or image_val.strip() == "":
                return CheckResult.UNKNOWN

            # If there's a digest, then this is even better than the tag, so the check passes
            if "@" in image_val:
                return CheckResult.PASSED

            re_result = re.findall(DOCKER_IMAGE_REGEX, image_val)
            if not re_result:
                return CheckResult.UNKNOWN
            (image, tag) = re_result[0]
            if tag == "latest" or tag == "":
                return CheckResult.FAILED
        else:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = ImageTagFixed()



from typing import Optional, List, Set

import pytest

from checkov.main import checkov_runners
from checkov.runner_filter import RunnerFilter


@pytest.mark.parametrize(
    "input_frameworks,input_skip_frameworks,expected_frameworks",
    [
        (["all"], None, {"all", "sast"}),
        (None, None, {"all"}),
        (["terraform"], None, {"terraform"}),
        (["cloudformation", "serverless"], None, {"cloudformation", "serverless"}),
        (["cdk"], None, {"cdk"}),
        (["cdk", "sast"], None, {"cdk", "sast"}),
        (
            ["all"],
            ["terraform", "secrets"],
            {
                "ansible",
                "argo_workflows",
                "arm",
                "azure_pipelines",
                "bicep",
                "cdk",
                "cloudformation",
                "dockerfile",
                "helm",
                "json",
                "yaml",
                "kubernetes",
                "serverless",
                "terraform_json",
                "terraform_plan",
                "github_configuration",
                "github_actions",
                "gitlab_configuration",
                "gitlab_ci",
                "bitbucket_configuration",
                "bitbucket_pipelines",
                "circleci_pipelines",
                "kustomize",
                "sca_package",
                "openapi",
                "sca_image",
                "sast",
                "3d_policy"
            },
        ),
        (["cloudformation", "serverless"], ["serverless", "secrets"], {"cloudformation"}),
    ],
    ids=["all", "none", "terraform", "multiple", "only cdk", "cdk and sast", "all_with_skip", "multiple_with_skip"],
)
def test_runner_filter_constructor_framework(
        input_frameworks: Optional[List[str]], input_skip_frameworks: Optional[List[str]], expected_frameworks: Set[str]
):
    # when
    runner_filter = RunnerFilter(
        framework=input_frameworks,
        runners=checkov_runners,
        skip_framework=input_skip_frameworks,
    )

    # then
    assert set(runner_filter.framework) == expected_frameworks



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AzureInstanceExtensions(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Virtual Machine Extensions are not Installed"
        id = "CKV_AZURE_50"
        supported_resources = ["Microsoft.Compute/virtualMachines"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/osProfile/allowExtensionOperations"

    def get_expected_value(self) -> bool:
        return False


check = AzureInstanceExtensions()



from __future__ import annotations

from typing import List, Tuple

from checkov.common.output.record import PLACEHOLDER_LINE
from checkov.sast.prisma_models.report import Flow


def get_data_flow_code_block(data_flow: List[Flow]) -> List[Tuple[int, str]]:
    code_block: List[Tuple[int, str]] = []

    prev_end = -1
    for flow in data_flow:
        if prev_end >= 0 and abs(prev_end - flow.start.row) > 1:
            code_block.append((-1, PLACEHOLDER_LINE))
        code_block.append((flow.start.row, flow.code_block + "\n"))
        prev_end = flow.end.row

    return cut_code_block_ident(code_block)


def get_code_block_from_start(lines: List[str], start: int) -> List[Tuple[int, str]]:
    code_block = [(index, line) for index, line in enumerate(lines, start=start)]
    return cut_code_block_ident(code_block)


def cut_code_block_ident(code_block: List[Tuple[int, str]]) -> List[Tuple[int, str]]:
    min_ident = len(code_block[0][1]) - len(code_block[0][1].lstrip())
    for item in code_block[1:]:
        current_min_ident = len(item[1]) - len(item[1].lstrip())
        if current_min_ident < min_ident:
            min_ident = current_min_ident

    if min_ident == 0:
        return code_block

    code_block_cut_ident = []
    for item in code_block:
        code_block_cut_ident.append((item[0], item[1][min_ident:]))
    return code_block_cut_ident



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GoogleComputeBlockProjectSSH(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure 'Block Project-wide SSH keys' is enabled for VM instances"
        id = "CKV_GCP_32"
        supported_resources = ['google_compute_instance', 'google_compute_instance_template',
                               'google_compute_instance_from_template']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if ('source_instance_template' in conf.keys() and 'metadata' not in conf.keys()) or \
                ('source_instance_template' in conf.keys() and isinstance(conf['metadata'][0], dict) and 'block-project-ssh-keys' not in
                 conf['metadata'][0].keys()):
            # if the source_instance_template value is there (indicating a google_compute_instance_from_template),
            # and block-project-ssh-keys is not present, then this check cannot PASS, since we don't know what the
            # underlying source template looks like.
            return CheckResult.UNKNOWN
        else:
            # in all other cases, pass/fail the check if block-project-ssh-keys is true/false or not present.
            return super().scan_resource_conf(conf)

    def get_inspected_key(self):
        return 'metadata/block-project-ssh-keys'


check = GoogleComputeBlockProjectSSH()



from __future__ import annotations

import logging
import os
import re
from typing import Tuple
from typing import Union, List, Any, Dict, Optional, TYPE_CHECKING

import igraph

from checkov.common.typing import LibraryGraph
from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_SEPARATOR_LENGTH, \
    TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR
from networkx import DiGraph

if TYPE_CHECKING:
    from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock

from checkov.common.util.type_forcers import force_int
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.variable_rendering.vertex_reference import TerraformVertexReference

MODULE_DEPENDENCY_PATTERN_IN_PATH = re.compile(r"\(\[\{.+\#\*\#.+\}\]\)")
CHECKOV_RENDER_MAX_LEN = force_int(os.getenv("CHECKOV_RENDER_MAX_LEN", "10000"))
CHECKOV_LOREM_IPSUM_VAL = '\x00'


def is_local_path(root_dir: str, source: str) -> bool:
    # https://www.terraform.io/docs/modules/sources.html#local-paths
    return (
        source.startswith(r"./")
        or source.startswith(r"/./")
        or source.startswith(r"../")
    )


def remove_module_dependency_in_path(path: str) -> Tuple[str, str, str]:
    """
    :param path: path that looks like "dir/main.tf[other_dir/x.tf#0]
    :return: separated path from module dependency: dir/main.tf, other_dir/x.tf
    """
    module_dependency = re.findall(MODULE_DEPENDENCY_PATTERN_IN_PATH, path)
    if re.findall(MODULE_DEPENDENCY_PATTERN_IN_PATH, path):
        path = re.sub(MODULE_DEPENDENCY_PATTERN_IN_PATH, "", path)
    module_and_num = extract_module_dependency_path(module_dependency)
    return path, module_and_num[0], module_and_num[1]


def extract_module_dependency_path(module_dependency: str | List[str]) -> List[str]:
    """
    :param: module_dependency: a list looking like ['[path_to_module.tf#0]']
    :return: the path without enclosing array and index: 'path_to_module.tf'
    """
    if not module_dependency:
        return ["", ""]
    if isinstance(module_dependency, list):
        module_dependency = module_dependency[0]
    return [
        module_dependency[3:module_dependency.index(f'.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}') + len('.tf')],
        module_dependency[module_dependency.index(f'.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}') + len(f'.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}'):-TERRAFORM_NESTED_MODULE_PATH_SEPARATOR_LENGTH]
    ]


BLOCK_TYPES_STRINGS = ("var", "local", "module", "data")
FUNC_CALL_PREFIX_PATTERN = re.compile(r"([.a-zA-Z]+)\(")
INTERPOLATION_EXPR = re.compile(r"\$\{([^\}]*)\}")
INDEX_PATTERN = re.compile(r"\[([0-9]+)\]")
MAP_ATTRIBUTE_PATTERN = re.compile(r"\[\"([^\d\W]\w*)\"\]")
NESTED_ATTRIBUTE_PATTERN = re.compile(r"\.\d+")


def get_vertices_references(
        str_value: str, aliases: Dict[str, Dict[str, str]], resources_types: List[str]
) -> List[TerraformVertexReference]:
    has_interpolation = True if "${" in str_value else False
    vertices_references: "list[TerraformVertexReference]" = []
    words_in_str_value = str_value.split()

    for word in words_in_str_value:
        if word.startswith((".", r"/.")):
            # check if word is a relative path
            continue

        if has_interpolation:
            interpolations = re.split(INTERPOLATION_EXPR, word)
            for interpolation_content in interpolations:
                add_vertices_references_from_word(
                    vertices_references=vertices_references,
                    word=interpolation_content,
                    aliases=aliases,
                    resources_types=resources_types,
                )
        else:
            add_vertices_references_from_word(
                vertices_references=vertices_references,
                word=word,
                aliases=aliases,
                resources_types=resources_types,
            )

    return vertices_references


def add_vertices_references_from_word(
    vertices_references: list[TerraformVertexReference],
    word: str,
    aliases: dict[str, dict[str, str]],
    resources_types: list[str],
) -> None:
    for w in word.split(","):
        word_sub_parts = w.split(".")
        if len(word_sub_parts) <= 1 or word_sub_parts[0].isnumeric():
            # if the word doesn't contain a '.' char, or if the first part before the dot is a number
            continue

        suspected_block_type = word_sub_parts[0]
        if suspected_block_type in BLOCK_TYPES_STRINGS:
            # matching cases like 'var.x'
            vertex_reference = TerraformVertexReference(
                block_type=suspected_block_type, sub_parts=word_sub_parts[1:], origin_value=w
            )
            if vertex_reference not in vertices_references:
                vertices_references.append(vertex_reference)
            continue

        vertex_reference_alias = get_vertex_reference_from_alias(suspected_block_type, aliases, word_sub_parts)
        if vertex_reference_alias and vertex_reference_alias not in vertices_references:
            vertex_reference_alias.origin_value = w
            # matching cases where the word is referring an alias
            vertices_references.append(vertex_reference_alias)
            continue

        # matching cases like 'aws_vpc.main'
        if word_sub_parts[0] in resources_types:
            block_name = word_sub_parts[0] + "." + word_sub_parts[1]
            word_sub_parts = [block_name] + word_sub_parts[2:]
            vertex_reference = TerraformVertexReference(
                block_type=BlockType.RESOURCE, sub_parts=word_sub_parts, origin_value=w
            )
            if vertex_reference not in vertices_references:
                vertices_references.append(vertex_reference)


def get_vertex_reference_from_alias(
        block_type_str: str, aliases: Dict[str, Dict[str, str]], val: List[str]
) -> Optional[TerraformVertexReference]:
    if not aliases:
        return None

    block_type = ""
    if block_type_str in aliases:
        block_type = aliases[block_type_str][CustomAttributes.BLOCK_TYPE]
    aliased_provider = ".".join(val)
    if aliased_provider in aliases:
        block_type = aliases[aliased_provider][CustomAttributes.BLOCK_TYPE]
    if block_type:
        return TerraformVertexReference(block_type=block_type, sub_parts=val, origin_value="")
    return None


def remove_function_calls_from_str(str_value: str) -> str:
    if "(" not in str_value:
        # otherwise it can't be a function call
        return str_value

    # remove start of function calls:: 'length(aws_vpc.main) > 0 ? aws_vpc.main[0].cidr_block : ${var.x}' --> 'aws_vpc.main) > 0 ? aws_vpc.main[0].cidr_block : ${var.x}'
    str_value = re.sub(FUNC_CALL_PREFIX_PATTERN, "", str_value)
    # remove ')'
    return str_value.replace(")", "")


def remove_index_pattern_from_str(str_value: str) -> str:
    if "[" not in str_value:
        # otherwise it can't be accessed via index
        return str_value

    str_value = re.sub(INDEX_PATTERN, "", str_value)
    str_value = str_value.replace('["', CHECKOV_LOREM_IPSUM_VAL).replace("[", " [ ").replace(CHECKOV_LOREM_IPSUM_VAL, '["')
    str_value = str_value.replace('"]', CHECKOV_LOREM_IPSUM_VAL).replace("]", " ] ").replace(CHECKOV_LOREM_IPSUM_VAL, '"]')
    return str_value


def remove_interpolation(str_value: str) -> str:
    if "${" not in str_value:
        # otherwise it can't be a string interpolation
        return str_value

    return str_value.replace("${", " ").replace("}", " ")


def replace_map_attribute_access_with_dot(str_value: str) -> str:
    if "[\"" not in str_value:
        # otherwise it can't be accessed via named index
        return str_value

    split_by_identifiers = re.split(MAP_ATTRIBUTE_PATTERN, str_value)
    new_split = []
    for split_part in split_by_identifiers:
        if split_part.startswith("."):
            split_part = split_part[1:]
        if split_part.endswith("."):
            split_part = split_part[:-1]
        new_split.append(split_part)

    return ".".join(new_split)


def get_referenced_vertices_in_value(
        value: Union[str, List[str], Dict[str, str]],
        aliases: Dict[str, Dict[str, str]],
        resources_types: List[str],
) -> List[TerraformVertexReference]:
    references_vertices: "list[TerraformVertexReference]" = []

    if not value or isinstance(value, (bool, int)):
        # bool/int values can't have a references to other vertices
        return references_vertices

    if isinstance(value, list):
        for sub_value in value:
            references_vertices += get_referenced_vertices_in_value(
                sub_value, aliases, resources_types
            )

    if isinstance(value, dict):
        for sub_value in value.values():
            references_vertices += get_referenced_vertices_in_value(
                sub_value, aliases, resources_types
            )

    if isinstance(value, str):
        references_vertices = get_referenced_vertices_in_str_value(
            str_value=value,
            aliases=aliases,
            resources_types=resources_types,
        )

    return references_vertices


def get_referenced_vertices_in_str_value(
    str_value: str,
    aliases: dict[str, dict[str, str]],
    resources_types: list[str],
) -> list[TerraformVertexReference]:
    references_vertices: "list[TerraformVertexReference]" = []

    value_len = len(str_value)
    if CHECKOV_RENDER_MAX_LEN and 0 < CHECKOV_RENDER_MAX_LEN < value_len:
        logging.debug(
            f'Rendering was skipped for a {value_len}-character-long string. If you wish to have it '
            f'evaluated, please set the environment variable CHECKOV_RENDER_MAX_LEN '
            f'to {str(value_len + 1)} or to 0 to allow rendering of any length'
        )
    else:
        if value_len < 5 or "." not in str_value:
            # the shortest reference is 'var.a' and references are done via dot notation
            return references_vertices

        str_value = remove_function_calls_from_str(str_value=str_value)
        str_value = remove_index_pattern_from_str(str_value=str_value)
        str_value = replace_map_attribute_access_with_dot(str_value=str_value)
        str_value = remove_interpolation(str_value=str_value)

        references_vertices = get_vertices_references(str_value, aliases, resources_types)

    return references_vertices


def generate_possible_strings_from_wildcards(origin_string: str, max_entries: int = 10) -> List[str]:
    max_entries = int(os.environ.get("MAX_WILDCARD_ARR_SIZE", max_entries))
    generated_strings = [origin_string]
    if not origin_string:
        return []
    if "*" not in origin_string:
        return generated_strings

    locations_of_wildcards = []
    for i, char in enumerate(origin_string):
        if char == "*":
            locations_of_wildcards.append(i)
    locations_of_wildcards.reverse()

    for wildcard_index in locations_of_wildcards:
        new_generated_strings = []
        for s in generated_strings:
            before_wildcard = s[:wildcard_index]
            after_wildcard = s[wildcard_index + 1:]
            for i in range(max_entries):
                new_generated_strings.append(before_wildcard + str(i) + after_wildcard)
        generated_strings = new_generated_strings

    # if origin_string == "ingress.*.cidr_blocks", check for "ingress.cidr_blocks" too
    generated_strings.append("".join(origin_string.split(".*")))
    return generated_strings


def attribute_has_nested_attributes(attribute_key: str, attributes: Dict[str, Any], attribute_is_leaf: Optional[Dict[str, bool]] = None) -> bool:
    """
    :param attribute_key: key inside the  `attributes` dictionary
    :param attributes:
    :return: True if attribute_key has inner attributes.
    Example 1: if attributes.keys == [key1, key.key2], type(attributes[key1]) is dict and return True for key1
    Example 2: if attributes.keys == [key1, key1.0], type(attributes[key1]) is list and return True for key1
    """
    if attribute_is_leaf is None:
        attribute_is_leaf = {}
    if attribute_is_leaf.get(attribute_key):
        prefixes_with_attribute_key = []
    else:
        prefixes_with_attribute_key = [a for a in attributes if a.startswith(attribute_key) and a != attribute_key]
    if not any(re.findall(NESTED_ATTRIBUTE_PATTERN, a) for a in prefixes_with_attribute_key):
        # if there aro no numeric parts in the key such as key1.0.key2
        return isinstance(attributes[attribute_key], dict)
    return isinstance(attributes[attribute_key], list) or isinstance(attributes[attribute_key], dict)


def attribute_has_dup_with_dynamic_attributes(attribute_key: str, attributes: dict[str, Any] | list[str]) -> bool:
    """
    :param attribute_key: key inside the `attributes` dictionary
    :param attributes: `attributes` dictionary
    :return: True if attribute_key has duplicate attribute with dynamic reference.
    :example: if attributes.keys == [name.rule, dynamic.name.content.rule] -> will return True.
    """
    attribute_key_paths = attribute_key.split('.')
    if len(attribute_key_paths) > 1:
        attar_key_dynamic_ref = f"dynamic.{attribute_key_paths[0]}.content.{attribute_key_paths[1]}"
        return attar_key_dynamic_ref in attributes
    else:
        return False


def get_related_resource_id(resource: dict[str, Any], file_path_to_referred_id: dict[str, str]) -> str | None:
    resource_id = resource.get(CustomAttributes.ID)
    # for external modules resources the id should start with the prefix module.[module_name]
    if resource.get(CustomAttributes.MODULE_DEPENDENCY):
        referred_id = file_path_to_referred_id.get(
            f'{resource.get(CustomAttributes.FILE_PATH)}[{resource.get(CustomAttributes.MODULE_DEPENDENCY)}#{resource.get(CustomAttributes.MODULE_DEPENDENCY_NUM)}]')
        resource_id = f'{referred_id}.{resource_id}'
    return resource_id


def get_file_path_to_referred_id_networkx(graph_object: DiGraph) -> dict[str, str]:
    file_path_to_module_id = {}

    modules = [node for node in graph_object.nodes.values() if
               node.get(CustomAttributes.BLOCK_TYPE) == BlockType.MODULE]
    for modules_data in modules:
        for module_name, module_content in modules_data.get(CustomAttributes.CONFIG, {}).items():
            for path in module_content.get("__resolved__", []):
                file_path_to_module_id[path] = f"module.{module_name}"
    return file_path_to_module_id


def get_file_path_to_referred_id_igraph(graph_object: igraph.Graph) -> dict[str, str]:
    file_path_to_module_id = {}
    modules = [v for v in graph_object.vs if
               v[CustomAttributes.BLOCK_TYPE] == BlockType.MODULE]
    for module_vertex in modules:
        module_name = module_vertex['name']
        module_content = module_vertex['attr'].get(CustomAttributes.CONFIG, {})
        for path in module_content.get('batch', {}).get("__resolved__", []):
            file_path_to_module_id[path] = f"module.{module_name}"
    return file_path_to_module_id


def get_file_path_to_referred_id_rustworkx(graph_object: DiGraph) -> dict[str, str]:
    file_path_to_module_id = {}

    modules = [node for index, node in graph_object.nodes() if
               node.get(CustomAttributes.BLOCK_TYPE) == BlockType.MODULE]
    for modules_data in modules:
        for module_name, module_content in modules_data.get(CustomAttributes.CONFIG, {}).items():
            for path in module_content.get("__resolved__", []):
                file_path_to_module_id[path] = f"module.{module_name}"
    return file_path_to_module_id


def setup_file_path_to_referred_id(graph_object: LibraryGraph) -> dict[str, str]:
    if isinstance(graph_object, igraph.Graph):
        return get_file_path_to_referred_id_igraph(graph_object)
    elif isinstance(graph_object, DiGraph):
        return get_file_path_to_referred_id_networkx(graph_object)
    else:
        return get_file_path_to_referred_id_rustworkx(graph_object)


def get_attribute_is_leaf(vertex: TerraformBlock) -> Dict[str, bool]:
    attribute_is_leaf = {}
    for attribute in vertex.attributes:
        attribute_is_leaf[attribute] = True
        other = '.'.join(attribute.split('.')[:-1])
        if other in attribute_is_leaf:
            attribute_is_leaf[other] = False
    return attribute_is_leaf


def join_double_quote_surrounded_dot_split(str_parts: list[str]) -> list[str]:
    """Joins back split strings which enclosed a dot by double quotes

    ex.

    ['google_project_iam_binding', 'role["roles/logging', 'admin"]'] -> ['google_project_iam_binding', 'role["roles/logging.admin"]']

    If someone finds a better solution feel free to replace it!
    """

    new_str_parts = []
    joined_str_parts: list[str] = []
    for part in str_parts:
        if not joined_str_parts:
            if '"' not in part:
                new_str_parts.append(part)
            elif part.count('"') >= 2:
                new_str_parts.append(part)
            else:
                joined_str_parts.append(part)
            continue

        joined_str_parts.append(part)

        if '"' in part:
            new_str_parts.append(".".join(joined_str_parts))
            joined_str_parts = []

    return new_str_parts



from dataclasses import dataclass
from typing import Optional


@dataclass
class ModuleParams:
    def __init__(
        self,
        root_dir: str,
        current_dir: str,
        source: str,
        source_version: Optional[str],
        dest_dir: str,
        external_modules_folder_name: str,
        inner_module: Optional[str] = None,
        tf_managed: bool = False,
    ):
        self.root_dir: str = root_dir
        self.current_dir: str = current_dir
        self.module_source: str = source
        self.version: Optional[str] = source_version
        self.dest_dir: str = dest_dir
        self.external_modules_folder_name: str = external_modules_folder_name
        self.inner_module: Optional[str] = inner_module
        self.tf_managed = tf_managed

        self.token: Optional[str] = None
        self.username: Optional[str] = None
        self.vcs_base_url: Optional[str] = None
        self.module_source_prefix: Optional[str] = None
        self.best_version: Optional[str] = None

        # terraform cloud / enterprise specific params
        self.tf_host_name: Optional[str] = None
        self.tf_modules_endpoint: Optional[str] = None
        self.tf_modules_versions_endpoint: Optional[str] = None



from abc import abstractmethod, ABC
from typing import List, Set
from checkov.common.output.report import Report
from checkov.common.sast.consts import SastLanguages
from checkov.sast.checks_infra.base_registry import Registry


class SastEngine(ABC):
    @abstractmethod
    def get_reports(self, targets: List[str], registry: Registry, languages: Set[SastLanguages]) -> List[Report]:
        pass



from __future__ import annotations

from typing import Any

from checkov.azure_pipelines.checks.base_azure_pipelines_check import BaseAzurePipelinesCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.yaml_doc.enums import BlockType


class SetSecretVariable(BaseAzurePipelinesCheck):
    def __init__(self) -> None:
        name = "Ensure set variable is not marked as a secret"
        id = "CKV_AZUREPIPELINES_3"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.SUPPLY_CHAIN,),
            supported_entities=("jobs[].steps[]", "stages[].jobs[].steps[]"),
            block_type=BlockType.ARRAY,
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        run_cmd = conf.get("bash") or conf.get("powershell")
        if run_cmd and isinstance(run_cmd, str):
            variable_found = False

            for line in run_cmd.splitlines():
                if "task.setvariable" in line:
                    variable_found = True

                    if "issecret=true" in line:
                        return CheckResult.FAILED, conf

            if variable_found:
                # should only pass, if it really found a set variable, otherwise unknown
                return CheckResult.PASSED, conf

        return CheckResult.UNKNOWN, conf


check = SetSecretVariable()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class CloudtrailDefinesSNSTopic(BaseResourceValueCheck):
    def __init__(self):
        """
        If you CloudTrail trails are not referenced to an SNS topic,
        you can't get notifications each time Amazon CloudTrail
        publishes any new log files, and you lose the ability to take realtime actions.

        AWS: "An active account can generate a large number of notifications. If you subscribe with email or SMS,
        you can receive a large volume of messages.
        We recommend that you subscribe using Amazon Simple Queue Service (Amazon SQS), which lets you
        handle notifications programmatically.
        """
        name = "Ensure CloudTrail defines an SNS Topic"
        id = "CKV_AWS_252"
        supported_resources = ['aws_cloudtrail']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        """
            Looks for SNS topic at cloudtrail:
            https://www.terraform.io/docs/providers/aws/r/cloudtrail.html
        :param conf: cloudtrail configuration
        :return: <CheckResult>
        """
        return 'sns_topic_name'

    def get_expected_value(self):
        return ANY_VALUE


check = CloudtrailDefinesSNSTopic()
